package ru.dbotthepony.mc.otm.item;

import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.stats.Stats;
import net.minecraft.tags.ItemTags;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.MenuProvider;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.event.entity.player.EntityItemPickupEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.eventbus.api.Event;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fmllegacy.network.NetworkEvent;
import net.minecraftforge.registries.RegistryManager;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.drive.DrivePool;
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive;
import ru.dbotthepony.mc.otm.capability.drive.MatteryDrive;
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.function.Supplier;

public class ItemPortableCondensationDrive extends Item {
	public ItemPortableCondensationDrive() {
		super(new Properties().stacksTo(1).tab(OverdriveThatMatters.CREATIVE_TAB));
	}

	@SubscribeEvent
	public static void onPickupEvent(EntityItemPickupEvent event) {
		if (
			event.getItem().getOwner() != null &&
			!event.getItem().getOwner().equals(event.getPlayer().getUUID()) &&
			event.getItem().getAge() < 200
		) {
			return;
		}

		var amount = event.getItem().getItem().getCount();
		var item = event.getItem().getItem().getItem();

		for (var stack : event.getPlayer().getInventory().items) {
			if (stack.getItem() instanceof ItemPortableCondensationDrive drive) {
				var _cap = stack.getCapability(MatteryCapability.DRIVE).resolve();

				if (_cap.isPresent()) {
					var cap = _cap.get();
					var filter = drive.getFilterSettings(stack);

					if (filter.matches(event.getItem().getItem())) {
						var copy = event.getItem().getItem().copy();
						var remaining = cap.insertItem(event.getItem().getItem(), false);

						if (remaining.getCount() == event.getItem().getItem().getCount()) {
							continue;
						}

						copy.setCount(copy.getCount() - remaining.getCount());

						event.setCanceled(true);

						net.minecraftforge.fmllegacy.hooks.BasicEventHooks.firePlayerItemPickupEvent(event.getPlayer(), event.getItem(), copy);

						if (remaining.getCount() > 0) {
							event.getItem().getItem().setCount(remaining.getCount());
							event.getPlayer().take(event.getItem(), amount - remaining.getCount());

							event.getPlayer().awardStat(Stats.ITEM_PICKED_UP.get(item), amount - remaining.getCount());
							event.getPlayer().onItemPickup(event.getItem());

							amount = remaining.getCount();
						} else {
							event.getPlayer().take(event.getItem(), amount);

							event.getPlayer().awardStat(Stats.ITEM_PICKED_UP.get(item), amount);
							event.getPlayer().onItemPickup(event.getItem());

							event.getItem().discard();
							return;
						}
					}
				}
			}
		}
	}

	private static class DriveCapability implements ICapabilityProvider {
		private UUID uuid;
		private final ItemStack stack;
		private final LazyOptional<IMatteryDrive> resolver = LazyOptional.of(() -> {
			return DrivePool.get(uuid, (tag) -> {
				var drive = new MatteryDrive(4000);
				drive.deserializeNBT(tag);
				return drive;
			}, () -> new MatteryDrive(4000));
		});

		DriveCapability(ItemStack stack) {
			this.stack = stack;
		}

		@Nonnull
		@Override
		public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
			if (cap == MatteryCapability.DRIVE) {
				if (uuid == null) {
					var get_value = stack.getOrCreateTag().getLongArray("uuid");

					if (get_value.length == 2) {
						uuid = new UUID(get_value[0], get_value[1]);
					} else {
						uuid = UUID.randomUUID();
						stack.getOrCreateTag().putLongArray("uuid", new long[] { uuid.getMostSignificantBits(), uuid.getLeastSignificantBits() });
					}
				}

				return resolver.cast();
			}

			return LazyOptional.empty();
		}
	}

	@Override
	public void appendHoverText(ItemStack stack, @Nullable Level p_41422_, List<Component> p_41423_, TooltipFlag p_41424_) {
		super.appendHoverText(stack, p_41422_, p_41423_, p_41424_);

		if (stack.getTag() != null) {
			var uuid = stack.getTag().getLongArray("uuid");

			if (uuid.length == 2) {
				p_41423_.add(new TextComponent(new UUID(uuid[0], uuid[1]).toString()).withStyle(ChatFormatting.GRAY));
			}
		}
	}

	@Nullable
	@Override
	public ICapabilityProvider initCapabilities(ItemStack stack, @Nullable CompoundTag nbt) {
		return new DriveCapability(stack);
	}

	public static class FilterSettings {
		public static final int MAX_FILTERS = 12;
		public final ItemStack[] items = new ItemStack[MAX_FILTERS];
		public boolean match_nbt;
		public boolean match_tag;
		public boolean blacklist;

		public FilterSettings() {
			Arrays.fill(items, ItemStack.EMPTY);
		}

		public FilterSettings(CompoundTag tag) {
			this();

			var list = tag.getList("filter", Tag.TAG_COMPOUND);
			var i = 0;

			for (var get_tag : list) {
				if (get_tag instanceof CompoundTag compound) {
					items[i++] = ItemStack.of(compound);
				}
			}

			match_nbt = tag.getBoolean("match_nbt");
			match_tag = tag.getBoolean("match_tag");
			blacklist = tag.getBoolean("blacklist");
		}

		public void serializeNBT(CompoundTag compound) {
			var list = new ListTag();

			for (int i = 0; i < MAX_FILTERS; i++) {
				list.add(items[i].serializeNBT());
			}

			compound.put("filter", list);
			compound.putBoolean("match_nbt", match_nbt);
			compound.putBoolean("match_tag", match_tag);
			compound.putBoolean("blacklist", blacklist);
		}

		public void serializeNBT(ItemStack drive) {
			serializeNBT(drive.getOrCreateTag());
		}

		public boolean matches(ItemStack stack) {
			if (blacklist) {
				for (var item : items) {
					if (!match_nbt && ItemStack.isSame(item, stack)) {
						return false;
					}

					if (match_tag) {
						var this_tags = item.getItem().getTags();
						var stack_tatgs = stack.getItem().getTags();

						for (var tag1 : this_tags) {
							if (stack_tatgs.contains(tag1)) {
								return false;
							}
						}
					}

					if (match_nbt && ItemStack.isSameItemSameTags(item, stack)) {
						return false;
					}
				}

				return true;
			} else {
				for (var item : items) {
					boolean same = ItemStack.isSame(item, stack);

					if (!same && match_tag) {
						var this_tags = item.getItem().getTags();
						var stack_tatgs = stack.getItem().getTags();

						for (var tag1 : this_tags) {
							if (stack_tatgs.contains(tag1)) {
								same = true;
								break;
							}
						}
					}

					if (match_nbt) {
						if (same && ItemStack.tagMatches(item, stack)) {
							return true;
						}
					} else {
						if (same) {
							return true;
						}
					}
				}

				return false;
			}
		}
	}

	public FilterSettings getFilterSettings(ItemStack drive) {
		return new FilterSettings(drive.getOrCreateTag());
	}
}
