package ru.dbotthepony.mc.otm.menu.widget;

import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.network.chat.Component;
import net.minecraft.world.inventory.ContainerData;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.screen.UVWindingOrder;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.function.Consumer;

/**
 * This widget class implement *shared* code
 * meaning these widgets are created inside Menu, not inside Screen
 * and these widgets are supposed to move data between server to client
 *
 * this is why this is different from Minecraft's clientside only widgets
 */
public abstract class AbstractWidget {
	/**
	 * Unlike other flags such as GaugeDirection, this tells how to finally draw the texture.
	 *
	 * This allows mirroring or flipping or both of drawn texture.
	 *
	 * @return uv order to use
	 */
	public UVWindingOrder getUVWindwing() {
		return UVWindingOrder.U0_V0_U1_V1;
	}

	public final MatteryMenu menu;
	public int x;
	public int y;
	protected Consumer<ContainerData> register_data;

	public boolean auto_parent = false;

	public abstract int getImageWidth();
	public abstract int getImageHeight();

	public abstract int getImageX();
	public abstract int getImageY();

	public AbstractWidget(@Nonnull MatteryMenu menu, int x, int y) {
		this.menu = menu;
		this.x = x;
		this.y = y;
		menu.addWidget(this, (consumer) -> register_data = consumer);
	}

	public AbstractWidget(@Nonnull MatteryMenu menu) {
		this(menu, 0, 0);
	}

	public AbstractWidget autoParent() {
		auto_parent = true;
		return this;
	}

	protected void addDataSlots(ContainerData slots) {
		register_data.accept(slots);
	}

	abstract public void updateServer();

	abstract public void renderBackground(PoseStack pose, float local_x, float local_y);

	// true - rendered a tooltip, break the loop
	// false - pass
	@Nullable
	abstract public List<Component> getTooltip();
}
