package ru.dbotthepony.mc.otm.screen.panels;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.narration.NarratableEntry;
import net.minecraft.client.gui.narration.NarrationElementOutput;
import net.minecraft.client.resources.sounds.SimpleSoundInstance;
import net.minecraft.network.chat.Component;
import net.minecraft.sounds.SoundEvents;
import ru.dbotthepony.mc.otm.screen.MatteryScreen;
import ru.dbotthepony.mc.otm.screen.RenderHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;

import static org.lwjgl.opengl.GL11.GL_ALWAYS;

public class FramePanel extends EditablePanel implements NarratableEntry {
	public enum FrameTabPosition {
		TOP(28, 28,28, 32),
		LEFT(28, 28, 32, 28),
		RIGHT(28, 28, 32, 28),
		BOTTOM(28, 28, 28, 32);

		public final int width;
		public final int height;

		public final int active_width;
		public final int active_height;

		FrameTabPosition(int width, int height, int active_width, int active_height) {
			this.width = width;
			this.height = height;
			this.active_width = active_width;
			this.active_height = active_height;
		}
	}

	public class FrameTabPanel extends EditablePanel {
		public final FrameTabPosition tab_position;

		public boolean isActive() {
			return active;
		}

		protected boolean active = false;
		protected boolean initial = false;

		protected Runnable on_open;
		protected Runnable on_close;

		public FrameTabPanel(FrameTabPosition position, Runnable on_open, Runnable on_close) {
			super(FramePanel.this.screen, FramePanel.this, 0, 0, position.width, position.height);
			tab_position = position;
			this.on_open = on_open;
			this.on_close = on_close;
		}

		public FrameTabPanel(FrameTabPosition position) {
			super(FramePanel.this.screen, FramePanel.this, 0, 0, position.width, position.height);
			tab_position = position;
		}

		public void bindOnOpen(Runnable value) {
			on_open = value;
		}

		public void bindOnClose(Runnable value) {
			on_close = value;
		}

		@Override
		protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			if (tab_position == FrameTabPosition.TOP) {
				int width;
				int height;

				if (active) {
					width = tab_position.active_width;
					height = tab_position.active_height;

					RenderHelper.window_background.render(stack, 2, 2, width - 4, height - 2);
				} else {
					width = tab_position.width;
					height = tab_position.height;

					RenderHelper.tab_background.render(stack, 2, 2, width - 4, height - 2);
				}

				RenderHelper.top_window_border.renderW(stack, 3, 0, width - 6);
				RenderHelper.left_window_border.renderH(stack, 0, 3, height - (active ? (initial ? 2 : 4) : 3));
				RenderHelper.right_window_border.renderH(stack, width - RenderHelper.right_window_border.rect_w(), 3, height - (active ? 4 : 3));

				RenderHelper.top_left_window_corner.render(stack, 0, 0);
				RenderHelper.top_right_window_corner.render(stack, width - RenderHelper.top_right_window_corner.rect_w(), 0);

				if (active) {
					if (!initial) {
						RenderHelper.tab_left_connection.render(stack, 0, height - RenderHelper.tab_left_connection.rect_h() - 1);
					}

					RenderHelper.tab_right_connection.render(stack, width - RenderHelper.tab_right_connection.rect_w(), height - RenderHelper.tab_left_connection.rect_h() - 1);
				}
			}
		}

		@Override
		protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
			if (active) {
				return true;
			}

			for (var tab : top_tabs) {
				if (tab != this) {
					if (tab.active) {
						tab.on_close.run();
						tab.active = false;
					}
				}
			}

			for (var tab : left_tabs) {
				if (tab != this) {
					if (tab.active) {
						tab.on_close.run();
						tab.active = false;
					}
				}
			}

			for (var tab : right_tabs) {
				if (tab != this) {
					if (tab.active) {
						tab.on_close.run();
						tab.active = false;
					}
				}
			}

			for (var tab : bottom_tabs) {
				if (tab != this) {
					if (tab.active) {
						tab.on_close.run();
						tab.active = false;
					}
				}
			}

			active = true;
			on_open.run();

			Minecraft.getInstance().getSoundManager().play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0F));
			return true;
		}
	}

	protected final ArrayList<FrameTabPanel> top_tabs = new ArrayList<>();
	protected final ArrayList<FrameTabPanel> left_tabs = new ArrayList<>();
	protected final ArrayList<FrameTabPanel> right_tabs = new ArrayList<>();
	protected final ArrayList<FrameTabPanel> bottom_tabs = new ArrayList<>();

	public FrameTabPanel addTab(FrameTabPosition position, Runnable on_open, Runnable on_close) {
		var tab = new FrameTabPanel(position, on_open, on_close);
		doAddTab(tab);
		return tab;
	}

	protected void doAddTab(FrameTabPanel tab) {
		if (top_tabs.size() == 0 && left_tabs.size() == 0 && right_tabs.size() == 0 && bottom_tabs.size() == 0) {
			tab.active = true;
		}

		switch (tab.tab_position) {
			case TOP -> top_tabs.add(tab);
			case LEFT -> left_tabs.add(tab);
			case RIGHT -> right_tabs.add(tab);
			case BOTTOM -> bottom_tabs.add(tab);
		}
	}

	public FrameTabPanel addTab(FrameTabPosition position) {
		var tab = new FrameTabPanel(position);
		doAddTab(tab);
		return tab;
	}

	@Override
	protected void performLayout() {
		for (int i = 0; i < top_tabs.size(); i++) {
			var tab = top_tabs.get(i);
			tab.setPos(i * FrameTabPosition.TOP.width, -FrameTabPosition.TOP.height);

			tab.initial = i == 0;
		}

		super.performLayout();
	}

	protected Component title;

	public static final int PADDING = 4;
	public static final int PADDING_TOP = 14;

	public FramePanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float width, float height, Component title) {
		super(screen, parent, x, y, width, height);
		this.title = title;
		setDockPadding(PADDING, PADDING_TOP, PADDING, PADDING);
	}

	protected boolean dragging = false;
	protected double last_mouse_x = 0;
	protected double last_mouse_y = 0;

	@Override
	protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
		var pos = screenToLocal(mouse_x, mouse_y);

		if (getParent() == null && pos.y() >= 0 && pos.y() <= 12) {
			setIgnoreMouseEventBoundaries(true);
			last_mouse_x = mouse_x;
			last_mouse_y = mouse_y;
			dragging = true;
			return true;
		}

		return super.mouseClickedInner(mouse_x, mouse_y, flag);
	}

	@Override
	protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
		if (dragging) {
			setIgnoreMouseEventBoundaries(false);

			dragging = false;
			return true;
		}

		return super.mouseReleasedInner(mouse_x, mouse_y, flag);
	}

	@Override
	protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int flag, double drag_x, double drag_y) {
		if (dragging) {
			var real_drag_x = mouse_x - last_mouse_x;
			var real_drag_y = mouse_y - last_mouse_y;

			last_mouse_x = mouse_x;
			last_mouse_y = mouse_y;

			setPos(getX() + real_drag_x, getY() + real_drag_y);
		}

		return super.mouseDraggedInner(mouse_x, mouse_y, flag, drag_x, drag_y);
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		// background
		// RenderSystem.disableDepthTest();
		RenderHelper.drawWindowBackground(stack, 0, 0, getWidth(), getHeight());
		// RenderSystem.enableDepthTest();

		// title
		RenderSystem.depthFunc(GL_ALWAYS);
		screen.getFont().draw(stack, title, 8, 5, 4210752);
		RenderSystem.depthFunc(GL_ALWAYS);
	}

	@Override
	public NarrationPriority narrationPriority() {
		return NarrationPriority.NONE;
	}

	@Override
	public void updateNarration(NarrationElementOutput narrationElementOutput) {

	}

	public void toScreenCenter() {
		setPos(screen.width / 2 - (int) (getWidth() / 2), screen.height / 2 - (int) (getHeight() / 2));
	}
}
